/****************************************************** 
 * File  : action.cc   
 * Desc. : functions for extracting actions from 
 *         universal plan
 * Author: Rune M. Jensen
 * Date  : 7/17/01
 ******************************************************/
#include <bdd.h>
#include <stream.h>
#include <math.h>
#include <set>
#include "execute.h"
#include "action.h"


bdd plan;

int en[4][3] =  // BDD var encoding of current vars; 
{ {7,9,11},  // hx
  {13,15,17},  // hy
  {19,21,23},  // px
  {25,27,29},  // py
};

int Hact[3] = {4,5,6};
 


void InitPlan(char *planname,int nodes,int cache) {

  // initialize bdd package 
  bdd_init(nodes,cache);
  bdd_setvarnum(100);
  
  
  // read bdd infile
  if (bdd_fnload(planname,plan))  {
    cout << "action.cc : InitPlan : Cannot load bdd from file \"" << planname << "\"\nexiting\n";
    exit(1);
  }
}


int Max(int a,int b) {
  if (a > b)
    return a;
  else
    return b;
}

int Min(int a,int b) {
  if (a > b)
    return b;
  else
    return a;
}

int Abs(int a) {
  if (a < 0)
    return -a;
  else
    return a;
}



action ChooseAct(gridstate st,int ManHunt, int ManPrey) {

  bdd actbdd;
  action res;

  // Choose hunter action by plan lookup
  // constrain universal plan by current state
  actbdd = plan;
  actbdd &= gridstate2bdd(st);


  // debug : Print out possible actions for the hunter 
  //cout << "Possible actions for hunter ";
  //for (int t=0; t < 5; t++)
  //  if ((int2bdd(Hact,3,t) & actbdd) != bddfalse )
  //    cout << t << " ";
  //cout << endl;


  if (ManHunt)
    {
      // Choose Hunt action from command line
      int HuntAct = -1;
      while (HuntAct < 0 || HuntAct > 8) {
	cout << "Hunt Move [0-4] : ";
	cin >> HuntAct;
      }
      res.Hunt = (enum HuntAct) HuntAct;
    }
  else
    {
      // choose Hunt action from plan
      int HuntAct = -1;
      int tryact = rand() % 5;
      while (HuntAct == -1) {
	if ( (int2bdd(Hact,3,tryact) & actbdd) != bddfalse )
	  HuntAct = tryact;
	tryact = rand() % 5;
      }
      
      res.Hunt = (enum HuntAct) HuntAct;
    }



  if (ManPrey)
    {
      // Choose prey action from command line
      int PreyAct = -1;
      while (PreyAct < 0 || PreyAct > 8) {
	cout << "Prey Move [0-8] : ";
	cin >> PreyAct;
      }
      res.Prey = (enum PreyAct) PreyAct;
    }
  else
    {
      // Move to chess board pos nearest hunter with different color as hunter
      // always possible
      
      // fill out a table: 
      // entry:actno,   steps from hunter (-1 if marked)
      int tbl[9];

      int tryact;      
      for (tryact = 0; tryact < 9; tryact++)
	{
	  int x,y;
	  x = st.p.x;
	  y = st.p.y;
	  
	  switch (tryact) { 
	    
	    // move_right 
	  case 0 :
	    x++;
	    break;
	    
	    // move_left
	  case 1 :
	    x--;
	    break;
	    
	    // move_up
	  case 2 :
	    y++;
	    break;
	    
	    // move_down
	  case 3 :
	    y--;
	    break;
	    
	    // move_right_up
	  case 4 :
	    x++;
	    y++;
	    break;
	    
	    // move_right_down
	  case 5 :
	    x++;
	    y--;
	    break;
	    
	    // move_left_up
	  case 6 :
	    x--;
	    y++;
	    break;
	
	    // move_left_down
	  case 7 :
	    x--;
	    y--;
	    break;
	    
	    // nop
	  case 8 :
	    break;
	  }
	  
	  // mark action if new position is outside the field
	  if (x > 7 || x < 0 || y > 7 || y < 0)
	    tbl[tryact] = -1;
	  else // mark pos if same color as hunter 
	    if ( (x + y) % 2 == (st.h.x + st.h.y) % 2 )
	      tbl[tryact] = -1;
	    else
	      {
		// find dist to hunter
		tbl[tryact] = Max(Abs(x - st.h.x),Abs(y - st.h.y));
	      }
	}
      
      // find min HuntDist
      int MinHuntDist = 100;
      for (tryact = 0; tryact < 9; tryact++)
	if (tbl[tryact] != -1)
	  MinHuntDist = Min(tbl[tryact],MinHuntDist);
      
      // mark entries above MinHuntDist    
      for (tryact = 0; tryact < 9; tryact++) 
	if ( tbl[tryact] > MinHuntDist )
	  tbl[tryact] = -1;
      
      // choose an action between the remaining actions
      int PreyAct = -1;
      tryact = rand() % 9;
      while (PreyAct == -1) {
	if ( tbl[tryact] != -1)
	  PreyAct = tryact;
	tryact = rand() % 9;
      }
      
      res.Prey = (enum PreyAct) PreyAct;
    }


  return res;
}







bdd gridstate2bdd(gridstate st) {
  
  bdd res = bddtrue;

  res &= int2bdd(en[0],3,st.h.x);
  res &= int2bdd(en[1],3,st.h.y);
  res &= int2bdd(en[2],3,st.p.x);
  res &= int2bdd(en[3],3,st.p.y);
  
  return res;
}
  

// IN
//   var    : bdd var mapping
//   length : number of bdd vars
//   n      : integer to encode.
//   OUT
//   bdd : BDD with the binary encoding of n
//         starting with the most significant bit. 
//         so if you write out the bdd vars 012345 the binary will
//         have its least sig. bit leftmost !!
//         All numbers are encoded by UMOP like this (also act. vars)
bdd int2bdd(int *var,int length,int n)  {
  bdd res;
  int j;  

  res = bddtrue;
  for (j=length-1; j >= 0; j--)
    {
      if (n % 2) 
	res &= bdd_ithvar(var[j]); 
      else
	res &= bdd_nithvar(var[j]);
      n /= 2;
    }
return res;
}









