/****************************************************** 
 * File  : draw.cc   
 * Desc. : main file for the UMOP excuter demo for
 *         hunter domain
 * Author: Rune M. Jensen
 * Date  : 7/17/01
 ******************************************************/

#include <X11/Xlib.h>

#include <stdio.h>
#include <stdlib.h>             /* getenv(), etc. */
#include <unistd.h>             /* sleep(), etc.  */
#include "main.h"
#include "draw.h"


WindowObj MkGrid(Display* display,int screen_size) {

  WindowObj res;

  int screen_num = DefaultScreen(display);

  // make the window
  res.win = XCreateSimpleWindow(display, RootWindow(display, screen_num),
                            0, 0,screen_size,screen_size, 2,
                            BlackPixel(display, screen_num),
                            WhitePixel(display, screen_num));

  res.width = screen_size;
  res.higth = screen_size;
  // make graphics Context for drawing in window

  unsigned long valuemask = 0;          /* which values in 'values' to  */
                                        /* check when creating the GC.  */
  XGCValues values;                     /* initial values for the GC.   */
  unsigned int line_width = 2;          /* line width for the GC.       */
  int line_style = LineSolid;           /* style for lines drawing and  */
  int cap_style = CapButt;              /* style of the line's edje and */
  int join_style = JoinBevel;           /*  joined lines.               */

  res.gc = XCreateGC(display, res.win, valuemask, &values);
  if (res.gc < 0) {
        fprintf(stderr, "XCreateGC: \n");
  }

  /* allocate foreground and background colors for this GC. */
  XSetForeground(display, res.gc, BlackPixel(display, screen_num));
  XSetBackground(display, res.gc, WhitePixel(display, screen_num));
  
  /* define the style of lines that will be drawn using this GC. */
  XSetLineAttributes(display, res.gc,
                     line_width, line_style, cap_style, join_style);

  /* define the fill style for the GC. to be 'solid filling'. */
  XSetFillStyle(display, res.gc, FillSolid);

  // insert win
  XMapWindow(display,res.win);
  XFlush(display);

  return res;
}





WindowObj MkRec(Display* display, Window parent,int width,int higth,int screen_size) {

  int screen_num = DefaultScreen(display);
  WindowObj res;

  res.higth = higth;
  res.width = width;


  // make window
  res.win = XCreateSimpleWindow(display,parent,screen_size/2,screen_size/2,width,higth,0,
				BlackPixel(display, screen_num),
				WhitePixel(display, screen_num));
  

  // make graphics Context for drawing in window

  unsigned long valuemask = 0;          /* which values in 'values' to  */
                                        /* check when creating the GC.  */
  XGCValues values;                     /* initial values for the GC.   */
  unsigned int line_width = 2;          /* line width for the GC.       */
  int line_style = LineSolid;           /* style for lines drawing and  */
  int cap_style = CapButt;              /* style of the line's edje and */
  int join_style = JoinBevel;           /* joined lines.                */

  res.gc = XCreateGC(display, res.win, valuemask, &values);
  if (res.gc < 0) {
        fprintf(stderr, "XCreateGC: \n");
  }

  /* allocate foreground and background colors for this GC. */
  XSetForeground(display, res.gc, BlackPixel(display, screen_num));
  XSetBackground(display, res.gc, WhitePixel(display, screen_num));
  
  /* define the style of lines that will be drawn using this GC. */
  XSetLineAttributes(display, res.gc,
                     line_width, line_style, cap_style, join_style);

  /* define the fill style for the GC. to be 'solid filling'. */
  XSetFillStyle(display, res.gc, FillSolid);

  // insert win
  XMapWindow(display,res.win);
  XFlush(display);

  return res;
}







void DrawGrid(Display* display, WindowObj grid,int screen_size) {

  // drawframe
  XDrawRectangle(display, grid.win, grid.gc,screen_size/10, screen_size/10,
    8*screen_size/10, 8*screen_size/10);

  // draw numbers
  for (int i=1; i<9; i++) {
    char str[16];
    sprintf(str,"%i",8-i);
    XDrawString(display,grid.win,grid.gc,screen_size/20, screen_size/20 + i*screen_size/10,str,1);
  }
  for (int i=1; i<9; i++) {
    char str[16];
    sprintf(str,"%i",i-1);
    XDrawString(display,grid.win,grid.gc,screen_size/20 + i*screen_size/10, screen_size - screen_size/20,str,1);
  }
  

  // draw chess board
  for (int x=0; x < 8; x++)
    for (int y=0; y < 8; y++)
      if ( (x + y) % 2 == 0)
	XFillRectangle(display,grid.win,grid.gc,(x+1)*screen_size/10,screen_size - screen_size/10*(y+2),screen_size/10,screen_size/10);
  

}



void RedrawGrid(Display* display, WindowObj grid, int screen_size) {

  // drawframe
  XDrawRectangle(display, grid.win, grid.gc,screen_size/10, screen_size/10,
    8*screen_size/10, 8*screen_size/10);

  // draw chess board
  for (int x=0; x < 8; x++)
    for (int y=0; y < 8; y++)
      if ( (x + y) % 2 == 0)
	XFillRectangle(display,grid.win,grid.gc,(x+1)*screen_size/10,screen_size - screen_size/10*(y+2),screen_size/10,screen_size/10);
  

}





void DrawRec(Display* display, WindowObj rec,char *colorname) {

  // get the colormap of the window
  XWindowAttributes wa;
  XGetWindowAttributes(display,rec.win,&wa);

  // lookup the color
  XColor color_exact;
  XColor color_possible;
  
  XLookupColor(display,wa.colormap,colorname,&color_exact,&color_possible);
  
  // get the pixel value of the possible color
  XAllocColor(display,wa.colormap,&color_possible);

  // set the GC foreground color to the returned pixel value
  XSetForeground(display,rec.gc,color_possible.pixel);

  // draw rec
  XFillRectangle(display,rec.win,rec.gc,0,0,rec.width,rec.higth);
}
















