/****************************************************** 
 * File  : execute.cc   
 * Desc. : execution module
 * Author: Rune M. Jensen
 * Date  : 7/17/01
 ******************************************************/
#include <X11/Xlib.h>
#include <stdlib.h>
#include "main.h"
#include "draw.h"
#include "action.h"
#include "execute.h"



// In 
// st          : state to execute action in
// act         : action to execute 
// speed       : simulation speed (1,2,...)
// screen_size : screen_size (1,2,...)
// Out
//  resulting state (side effect visual execution of joint action)
gridstate execute(DisplayInfo di, gridstate old_st, action act, int speed) {
  
  gridstate new_st;

  // compute new state
  new_st = old_st;

  switch (act.Hunt) {  
    
  case move_right_upH :
    new_st.h.x++;
    new_st.h.y++;
    break;
    
  case move_right_downH :
    new_st.h.x++;
    new_st.h.y--;
    break;
    
  case move_left_upH :
    new_st.h.x--;
    new_st.h.y++;
    break;
    
  case move_left_downH :
    new_st.h.x--;
    new_st.h.y--;
    break;
    
  case nopH :
    break;
  }
  

  switch (act.Prey) {  
  case move_right :
    new_st.p.x++;
    break;
    
  case move_left :
    new_st.p.x--;
    break;
    
  case move_up :
    new_st.p.y++;
    break;
    
  case move_down :
    new_st.p.y--;
    break;
    
  case move_right_up :
    new_st.p.x++;
    new_st.p.y++;
    break;
    
  case move_right_down :
    new_st.p.x++;
    new_st.p.y--;
    break;
    
  case move_left_up :
    new_st.p.x--;
    new_st.p.y++;
    break;
    
  case move_left_down :
    new_st.p.x--;
    new_st.p.y--;
    break;
    
  case nop :
    break;
  }
  

  
  // simulate change
  Simulate(di,old_st,new_st,speed);
  

  return new_st;
}

 

gridstate initstate(void) {
  
  gridstate res;
 
  
  res.h.x = rand() % 8;
  res.h.y = rand() % 8;

  res.p.x = rand() % 8;
  res.p.y = rand() % 8;

  while (res.h.x == res.p.x && res.h.y == res.p.y) {
    res.h.x = rand() % 8;
    res.h.y = rand() % 8;
    
    res.p.x = rand() % 8;
    res.p.y = rand() % 8;
  }

  return res;
}
    
    

bool goal_reached(gridstate st) {
  return (st.h.x == st.p.x && st.h.y == st.p.y);
}


void Move2GridState(DisplayInfo di,gridstate st) {

  screenstate sst;

  sst.h = Grid2ScreenPos(st.h,di.screen_size);
  sst.p = Grid2ScreenPos(st.p,di.screen_size);

  // move things around
  
  XMoveWindow(di.display,di.Hunter.win,int(sst.h.x + di.screen_size/120),int(sst.h.y + di.screen_size/120));
  XMoveWindow(di.display,di.Prey.win,int(sst.p.x + di.screen_size/120),int(sst.p.y + di.screen_size/120));

  DrawRec(di.display,di.Hunter,"gold");
  DrawRec(di.display,di.Prey,"RoyalBlue");


  /* redraw grid */
  DrawGrid(di.display,di.grid,di.screen_size);

  XFlush(di.display);

}
  


void Simulate(DisplayInfo di,gridstate old_st,gridstate new_st,int speed) {

  screenstate old_sst;
  screenstate d_sst;

  old_sst.h = Grid2ScreenPos(old_st.h,di.screen_size);
  old_sst.p = Grid2ScreenPos(old_st.p,di.screen_size);

  d_sst.h = DeltaScreenPos(old_st.h,new_st.h,di.screen_size);
  d_sst.p = DeltaScreenPos(old_st.p,new_st.p,di.screen_size);
  
  // move things around
  for (int i = 1; i < di.screen_size/(10*speed); i = i + speed) {

    XMoveWindow(di.display,di.Hunter.win,int(old_sst.h.x + i*d_sst.h.x + di.screen_size/120),
		int(old_sst.h.y + i*d_sst.h.y + di.screen_size/120));
    XMoveWindow(di.display,di.Prey.win,int(old_sst.p.x + i*d_sst.p.x + di.screen_size/120),
		int(old_sst.p.y + i*d_sst.p.y + di.screen_size/120));

    DrawRec(di.display,di.Hunter,"gold");
    DrawRec(di.display,di.Prey,"RoyalBlue");
    

    /* draw grid */
    DrawGrid(di.display,di.grid,di.screen_size);
    
    XFlush(di.display);
    
  }
}

 


screenpos Grid2ScreenPos(gridpos gp,int screen_size) {

  screenpos res;

  res.x = (1 + gp.x) * screen_size/10;
  res.y = screen_size - (2 + gp.y)*screen_size/10 ;

  return res;
}


screenpos DeltaScreenPos(gridpos old_p, gridpos new_p, int screen_size) {

  screenpos n_p;
  screenpos o_p;
  screenpos res;
  n_p = Grid2ScreenPos(new_p,screen_size);
  o_p = Grid2ScreenPos(old_p,screen_size);

  res.x = double(n_p.x - o_p.x) / double(screen_size/10);
  res.y = double(n_p.y - o_p.y) / double(screen_size/10);
  
  return res;
}











