/****************************************************** 
 * File  : main.cc   
 * Desc. : main file for the UMOP excuter demo for
 *         knight hunter domain
 * Author: Rune M. Jensen
 * Date  : 7/21/01
 ******************************************************/

#include <X11/Xlib.h>

#include <stdio.h>
#include <stdlib.h>             /* getenv(), etc. */
#include <stream.h>
#include <unistd.h>             /* sleep(), etc.  */

#include "main.h"
#include "draw.h"
#include "execute.h"
#include "action.h"


int main(int argc, char* argv[]) {

  // default values of input parameters
  char planname[256];
  int nodes = 500000;
  int cache = 10000;
  int size   = 10;
  int speed  = 1;
  int seed   = 998357;
  int ManHunt  = 0;
  int ManPrey  = 0;

  planname[0]  = 0;

 
  /*** Scan command line ***/
  for (int ind = 1; ind < argc; ind++) {
    
    /*** Parse switches ***/
    if (argv[ind][0] == '-') {
      switch (argv[ind][1]) {  
      case 'i': strcpy(planname,argv[++ind]);
	break;
      case 'n': nodes = atoi(argv[++ind]);
	break;
      case 'c': cache = atoi(argv[++ind]);
	break;
      case 's': size = atoi(argv[++ind]);
	break;
      case 'v': speed = atoi(argv[++ind]);
	break;
      case 'h': ManHunt = 1;
	break;
      case 'p': ManPrey = 1;
	break;
      case 'r': seed = atoi(argv[++ind]);
	break;
      case '?':
        cout << " i input planname \n"; 
        cout << " n number of packages nodes \n"; 
        cout << " c cache size \n"; 
	cout << " s size \n";
	cout << " v speed \n";
	cout << " r seed \n";
        cout << " h Hunter move entered by user \n";
        cout << " p Prey move entered by user \n";
	cout << " ? usage \n";
	exit(1);
        break;
      default : printf("main.cc main : Unknown switch '%c'\nexiting\n", argv[ind][1]);
	exit(1);
	break;
      }
    }
  }
  
  // Set input parameters
  if (planname[0] == 0) {
    strcpy(planname,"hunter.plan");
  }

  // read universal plan
  InitPlan(planname,nodes,cache);


  // open connection to the X server. 
  Display* display;                        
  char *display_name = getenv("DISPLAY");  

  display = XOpenDisplay(display_name);
  if (display == NULL) {
    fprintf(stderr, "%s: cannot connect to X server '%s'\n",
            argv[0], display_name);
    exit(1);
  }


  // make obj windows (and pack it all down in a displayinfo)
  DisplayInfo di;

  di.display = display;
  di.screen_size = size*WIN_SIZE;
  di.grid = MkGrid(display,di.screen_size); 
  di.Hunter = MkRec(display,di.grid.win,di.screen_size/12,di.screen_size/12,di.screen_size);
  di.Prey = MkRec(display,di.grid.win,di.screen_size/12,di.screen_size/12,di.screen_size);


  // initialize simulation
  srand(seed);
  gridstate st;
  st = initstate();
  
  // show initstate
  Move2GridState(di,st);
  
  // simulate
  action act;
  
  int StepNum = 0;
  int KillNum = 0;

  while (1) { 
    
    StepNum++;
   
    // read joint action from plan
    act = ChooseAct(st,ManHunt,ManPrey);
    
    // execute it (and get new state)
    st = execute(di,st,act,speed);

    // start over if goal reached
    if (goal_reached(st)) {
      KillNum++;
      cout << "Total StepNum = " << StepNum << endl;
      cout << "Total KillNum = " << KillNum << endl;
      cout << "Average StepNum per Kill = " << StepNum / KillNum << endl << endl; 
      st = initstate();
      Move2GridState(di,st);
    }
  } 

  return 0;
}




void  printusage() {
  printf("USAGE:\n");
  printf("       -i <plan name>.plan      (default \"magnet.plan\")\n");
  printf("       -s size                  (default 1)\n");
  printf("       -v velocity              (default 1)\n");
  printf("       -r random seed           (default 98476)\n");
  printf("\n");
}
