/****************************************************** 
 * File  : main.cc   
 * Desc. : main file for the UMOP excuter demo for
 *         magnet domain
 * Author: Rune M. Jensen
 * Date  : 4/4/01
 ******************************************************/

#include <X11/Xlib.h>

#include <stdio.h>
#include <stdlib.h>             /* getenv(), etc. */
#include <stream.h>
#include <unistd.h>             /* sleep(), etc.  */

#include "main.h"
#include "draw.h"
#include "execute.h"
#include "action.h"


int main(int argc, char* argv[]) {

  // default values of input parameters
  char planname[256];
  int size   = 10;
  int speed  = 1;
  int seed   = 998357;

  planname[0]  = 0;

 
  /*** Scan command line ***/
  for (int ind = 1; ind < argc; ind++) {
    
    /*** Parse switches ***/
    if (argv[ind][0] == '-') {
      switch (argv[ind][1]) {  
      case 'p': strcpy(planname,argv[++ind]);
	break;
      case 's': size = atoi(argv[++ind]);
	break;
      case 'v': speed = atoi(argv[++ind]);
	break;
      case 'r': seed = atoi(argv[++ind]);
	break;
      case 'h':
        cout << " p planname \n"; 
	cout << " s size \n";
	cout << " v speed \n";
	cout << " r seed \n";
	exit(1);
        break;
      default : printf("main.cc main : Unknown switch '%c'\nexiting\n", argv[ind][1]);
	exit(1);
	break;
      }
    }
  }
  
  // Set input parameters
  if (planname[0] == 0) {
    strcpy(planname,"magnet.plan");
  }

  // read universal plan
  InitPlan(planname);


  // open connection to the X server. 
  Display* display;                        
  char *display_name = getenv("DISPLAY");  

  display = XOpenDisplay(display_name);
  if (display == NULL) {
    fprintf(stderr, "%s: cannot connect to X server '%s'\n",
            argv[0], display_name);
    exit(1);
  }


  // make obj windows (and pack it all down in a displayinfo)
  DisplayInfo di;

  di.display = display;
  di.screen_size = size*WIN_SIZE;
  di.grid = MkGrid(display,di.screen_size); 
  di.Obj1 = MkRec(display,di.grid.win,di.screen_size/12,di.screen_size/12,di.screen_size);
  di.Obj2 = MkRec(display,di.grid.win,di.screen_size/12,di.screen_size/12,di.screen_size);
  di.BarV = MkRec(display,di.grid.win,di.screen_size/40,8*di.screen_size/10,di.screen_size);
  di.BarH = MkRec(display,di.grid.win,8*di.screen_size/10,di.screen_size/40,di.screen_size);


  // initialize simulation
  srand(seed);
  gridstate st;
  st = initstate();
  
  // show initstate
  Move2GridState(di,st);
  
  // simulate
  action act;
  
  while (1) { 
   
    // read joint action from plan
    act = ChooseAct(st);
    
    // execute it (and get new state)
    st = execute(di,st,act,speed);

    // start over if goal reached
    if (goal_reached(st)) {
      st = newinitstate(st);
      Move2GridState(di,st); 
    }
  } 
  return 0;
}




void  printusage() {
  printf("USAGE:\n");
  printf("       -p <plan name>.plan      (default \"magnet.plan\")\n");
  printf("       -s size                  (default 1)\n");
  printf("       -v velocity              (default 1)\n");
  printf("       -r random seed           (default 98476)\n");
  printf("\n");
}
