/*========================================================================
                              
                                ** UMOP Executor **

            Copyright (C) 1999 by Rune M. Jensen and Manuela M. Veloso
            Computer Science Department, Carnegie Mellon University.
            
    Permission is hereby granted to distribute the UMOP software for 
    research purposes, provided this notice is included with any such 
    distribution. 

      *** WE ARE NOT RESPONSIBLE FOR ANY KIND OF DAMAGE TO YOUR  ***
      *** FILES, DATA, HARDWARE, LOSS OF MONEY, SYSTEM CRASHES   *** 
      *** OR ANY OTHER THING YOU MIGHT COME UP WITH.             ***
      *** - USE THIS PROGRAM OF YOUR OWN FREE WILL !!!           ***

========================================================================*/
/****************************************************** 
 * File  : exefunc.cc   
 * Desc. : functions for extracting actions from 
 *         universal plan
 * Author: Rune M. Jensen
 * Date  : 10/16/01
 ******************************************************/
#include <bdd.h>
#include <stream.h>
#include <math.h>
#include <set>
#include "fsm.hpp"
#include "exefunc.hpp"
#include "bddprint.hpp" 


// finds the inital state of the trace
bdd chooseInit(domdef *def,bdd init, int mktrace, FILE *trace) {

  bdd initState = bddtrue;
  int value;
  
  printf("Please enter the initial state of the trace\n");
  if (mktrace) fprintf(trace,"The initial state of trace is:\n");

  // go through the state variables and assign a value
  for (int i = 0; i < def->varnames.size(); i++)
    {
      vardef v(def->varmap[def->varnames[i]]);
      
      int range;
      if (v.type == vt_bool)
	range = 1;
      else
	range = int(pow(2,v.length)) - 1;
      
      int outofrange = 1;
      while (outofrange)
	{
	  printf("  %s [0-%d] = ",v.name.c_str(),range);
	  scanf("%d",&value);
	  if (value < 0 || value > range)
	    printf("    Value out of range, re-enter value !\n");
	  else
	    outofrange = 0;
	}
      
      // print and add value to init state
      if (mktrace)  fprintf(trace,"  %s = %d\n",v.name.c_str(),value);
      initState &= int2bdd(v.var,v.length,value);
    }

  if ( (initState & init) == bddfalse ) 
    {
      printf("Warning: initial state of the trace is not a member of the initial states of the plan\n");
      if (mktrace)
	fprintf(trace,"Initial state of the trace is not a member of the initial states of the plan\n");
    }

  return initState;
}




// simple prints a set of integers as "5,12,56,6" and so on
void printintset(set<int> intset, FILE *out) {
  for (set<int>::iterator i = intset.begin(); i != intset.end(); ++i)
    {
      set<int>::iterator next;
      next = i;
      ++next;
      fprintf(out,"%d",*i);
      if (next != intset.end()) fprintf(out,",");
    }
}


// makes a conjunction with a partitioned transition relation
bdd conjunc(trel *T, bdd state) {
  
  bdd res = state; 

  for (int i=0; i < T->partnum; i++)
    res &= T->p[i].exp; 

  return res;
}



// for agt agtno find the set of applicable actions
// in currentstate
set<int> appagtacts(domdef *def, trel *T, bdd currentstate,int agtno) {

  set<int> res;

  int actnum = def->agt[agtno].act.size();

  bdd cs = conjunc(T,currentstate);
  
  // go through each action and determine if 
  // it is applicable
  vardef v(def->var[agtno]);

  for (int i=0; i<actnum; i++)
    if ( (cs & int2bdd(v.var,v.length,i)) != bddfalse ) res.insert(i);

  return res;
}    

      

// for system agent agtno, find the set of actions in the plan
set<int> plansysacts(domdef *def, bdd plan, bdd currentstate, int agtno) {

  set<int> res;
  
  int actnum = def->agt[agtno].act.size();

  bdd cs = plan & currentstate;
  
  // go through each action and determine if 
  // it is applicable
  vardef v(def->var[agtno]);

  for (int i=0; i<actnum; i++)
    if ( (cs & int2bdd(v.var,v.length,i)) != bddfalse ) res.insert(i);

  return res;
}


// in currentstate choose the joint action of the sysetm agents
bdd chooseSysAct(domdef *def, trel *T, bdd plan, bdd currentstate, int sysmanual, int mktrace, FILE *trace) {
  bdd sysact = bddtrue;

  set<int>  appacts;
  set<int>  planacts;

  if (sysmanual)
    {
      printf("\n  Choose joint action of system agents\n");
      if (mktrace) fprintf(trace,"\n  Joint action of system agents:\n");
      
      // go through system agents
      for (int i = def->envnum; i < def->agt.size() ; i++)
	{
	  appacts = appagtacts(def,T,currentstate,i);

	  if (appacts.empty())
	    {
	      printf("\n  No applicable actions for agent %s, ending trace!\n",def->agt[i].name.c_str());
	      if (mktrace)  fprintf(trace,"  No applicable actions for agent %s, ending trace!\n",def->agt[i].name.c_str());
	      return bddfalse;
	    }



          planacts = plansysacts(def,plan,currentstate,i);
	  int actnotapp = 1;
	  while (actnotapp)
	    {
	      printf("\n    Agt %s\n",def->agt[i].name.c_str());
	      printf("     Action names:");
	      for (int j=0; j < def->agt[i].act.size(); j++)
		printf(" %d=%s",j,def->agt[i].act[j].name.c_str());
	      printf("\n     Applicable actions: ");
	      printintset(appacts,stdout);
	      printf("\n     Actions in plan: ");
	      printintset(planacts,stdout);
	      printf("\n     Choose action : ");
	      int cact;
	      scanf("%d",&cact);
	      if (appacts.find(cact) == appacts.end())
		printf("\n     Action is not applicable in the currentstate, re-enter action !!!\n\n");
	      else
		{
		  actnotapp = 0;
		  if (mktrace) fprintf(trace,"     action of %s=%s\n",
				       def->agt[i].name.c_str(), 
				       def->agt[i].act[cact].name.c_str());
		  
		  vardef v(def->var[i]);
		  sysact &= int2bdd(v.var,v.length,cact);
		  printf("\n");
		}	      
	    }
	}
    }
  else // action pick is automatic
    {
      printf("\n  Joint action of system agents:\n");
      if (mktrace) fprintf(trace,"\n  Joint action of system agents:\n");

      // go through system agents
      for (int i=def->envnum; i < def->agt.size(); i++)
	{	
          planacts = plansysacts(def,plan,currentstate,i);

	  if (planacts.empty())
	    {
	      printf("\n  No plan actions for agent %s, ending trace!\n\n",def->agt[i].name.c_str());
	      if (mktrace)  fprintf(trace,"\n  No plan actions for agent %s, ending trace!\n\n",def->agt[i].name.c_str());
	      return bddfalse;
	    }

	  int choice = rand() % planacts.size();
	  set<int>::iterator si;
	  si = planacts.begin();
	  for (int j=0; j < choice; j++)
	    ++si;
	  printf("     action of %s=%s\n",
		 def->agt[i].name.c_str(), 
		 def->agt[i].act[*si].name.c_str());	  
	  if (mktrace) fprintf(trace,"     action of %s=%s\n",
			       def->agt[i].name.c_str(), 
			       def->agt[i].act[*si].name.c_str());
	  
	  vardef v(def->var[i]);
	  sysact &= int2bdd(v.var,v.length,*si);
	  printf("\n");
	}
    }
  return sysact;
}



bdd chooseEnvAct(domdef *def, trel *T, bdd plan, bdd currentstate, int envmanual, int mktrace, FILE *trace) {

  bdd envact = bddtrue;

  set<int>  appacts;

  if (envmanual)
    {
      printf("  \nChoose joint action of environment agents\n");
      if (mktrace) fprintf(trace,"  \nJoint action of environment agents:\n");
      
      // go through environment agents
      for (int i=0; i < def->envnum; i++)
	{
	  appacts = appagtacts(def,T,currentstate,i);
	  
	  if (appacts.empty())
	    {
	      printf("\n  No applicable actions for agent %s, ending trace!\n\n",def->agt[i].name.c_str());
	      if (mktrace)  fprintf(trace,"\n  No applicable actions for agent %s, ending trace!\n\n",def->agt[i].name.c_str());
	      return bddfalse;
	    }

	  int actnotapp = 1;
	  while (actnotapp)
	    {
	      printf("\n    Agt %s\n",def->agt[i].name.c_str());
	      printf("     Action names:");
	      for (int j=0; j < def->agt[i].act.size(); j++)
		printf(" %d=%s",j,def->agt[i].act[j].name.c_str());
	      printf("\n     Applicable actions: ");
	      printintset(appacts,stdout);
	      printf("\n     Choose action : ");
	      int cact;
	      scanf("%d",&cact);
	      if (appacts.find(cact) == appacts.end())
		printf("\n     Action is not applicable in the currentstate, re-enter action !!!\n\n");
	      else
		{
		  actnotapp = 0;
		  if (mktrace) fprintf(trace,"     action of %s=%s\n",
				       def->agt[i].name.c_str(), 
				       def->agt[i].act[cact].name.c_str());
		  
		  vardef v(def->var[i]);
		  envact &= int2bdd(v.var,v.length,cact);
		  printf("\n");
		}	      
	    }
	}
    }
  else // action pick is automatic
    {
      printf("\n  Joint action of environment agents:\n");
      if (mktrace) fprintf(trace,"\n  Joint action of environment agents:\n");

      // go through system agents
      for (int i=0; i < def->envnum; i++)
	{	
	  appacts = appagtacts(def,T,currentstate,i);

	  if (appacts.empty())
	    {
	      printf("\n  No applicable actions for agent %s, ending trace!\n\n",def->agt[i].name.c_str());
	      if (mktrace)  fprintf(trace,"\n  No applicable actions for agent %s, ending trace!\n\n",def->agt[i].name.c_str());
	      return bddfalse;
	    }


	  int choice = rand() % appacts.size();
	  set<int>::iterator si;
	  si = appacts.begin();
	  for (int j=0; j < choice; j++)
	    ++si;
	  printf("     action of %s=%s\n",
		 def->agt[i].name.c_str(), 
		 def->agt[i].act[*si].name.c_str());	  
	  if (mktrace) fprintf(trace,"     action of %s=%s\n",
			       def->agt[i].name.c_str(), 
			       def->agt[i].act[*si].name.c_str());
	  
	  vardef v(def->var[i]);
	  envact &= int2bdd(v.var,v.length,*si);
	  printf("\n");
	}
    }
  return envact;
}




set<int> possiblevalues(vardef v,int range,bdd nextstate) {
  
  set<int> res;

  for (int i=0; i <= range; i++)
    if ( ( nextstate & int2bdd(v.var,v.length,i)) != bddfalse )
      res.insert(i);
  
  return res;
}



bdd chooseNewState(domdef *def, bdd envact, bdd sysact, trel *T, bdd currentstate, int mktrace, FILE *trace) {

  bdd nextstate;
    
  //constrain T with sys and env actions
  nextstate =  conjunc(T,envact & sysact & currentstate);

  // get rid of actions and current state
  nextstate = bdd_exist(nextstate,def->rels->sysactions);
  nextstate = bdd_exist(nextstate,def->rels->envactions);
  nextstate = bdd_exist(nextstate,def->rels->oldstatefull);

  // replace new with old
  nextstate = bdd_replace(nextstate,def->rels->new2old);

  // go through the state variables and assign a value
  printf("\n  Currentstate:\n  ");
  if (mktrace) fprintf(trace,"  Currentstate:\n  ");

  for (int i = 0; i < def->varnames.size(); i++)
    {
      vardef v(def->varmap[def->varnames[i]]);
      
      int range;
      if (v.type == vt_bool)
	range = 1;
      else
	range = int(pow(2,v.length)) - 1;

      set<int> pv = possiblevalues(v,range,nextstate);
      
      // pick a random value
      
      int choice = rand() % pv.size();
      set<int>::iterator si;
      si = pv.begin();
      for (int j=0; j < choice; j++)
	++si;
     
      printf(" %s=%d",v.name.c_str(),*si);
      if (mktrace) fprintf(trace," %s=%d",v.name.c_str(),*si);

      nextstate &= int2bdd(v.var,v.length,*si);
    }

  printf("\n");
  if (mktrace) fprintf(trace,"\n");

  return nextstate;
}



































