/*========================================================================
                              
                                ** UMOP **
		** Universal Multi-Agent OBDD-based Planner **

            Copyright (C) 1999 by Rune M. Jensen and Manuela M. Veloso
            Computer Science Department, Carnegie Mellon University.
            
    Permission is hereby granted to distribute the UMOP software for 
    research purposes, provided this notice is included with any such 
    distribution. 

      *** WE ARE NOT RESPONSIBLE FOR ANY KIND OF DAMAGE TO YOUR  ***
      *** FILES, DATA, HARDWARE, LOSS OF MONEY, SYSTEM CRASHES   *** 
      *** OR ANY OTHER THING YOU MIGHT COME UP WITH.             ***
      *** - USE THIS PROGRAM OF YOUR OWN FREE WILL !!!           ***

========================================================================*/
/******************************************************
 * File  : fsm.hpp
 * Desc. : Structures for constructing the FSM 
 *         transition relation for MNP problems.
 * Author: Rune M. Jensen, CS, CMU, (IAU, DTU)
 * Date  : 3/3/99
 ******************************************************/

#ifndef FSMHPP
#define FSMHPP

#include <bdd.h>
#include <fstream.h>
#include <string>
#include <set>
#include <vector>
#include <list>
#include <map>
#include "common.hpp"
#include "domain.hpp"

/****************************************************
 * structures for representing domain info          *
 ****************************************************/
						      

//*************  agtdef    ****************************

enum sysclass {sc_env, sc_sys};

struct agtdef {
  string name;
  sysclass sclass;
  vector<action>  act;
  agtdef(string n, sysclass sc, vector<action> ac)
    { name = n; sclass = sc; act = ac; }
  void print();
};

//*************  vardef    ****************************


struct  vardef {
  string     name;
  vtype      type;     // boolean, scalar
  int        length;   // num of BDD variables
  int        *var;     // mapping to bddvars 
  vardef(string n,vtype vt,int l, int *v)
     { name =n; type = vt; length = l; var = v;}	
  vardef()  // default value needed for map
     { name = ""; type = vt_bool; length = 0; var = NULL; }
  void print();
};


//       bdd variable location info                 
struct varrels {
  bddPair *old2new;
  bddPair *new2old;
  bdd     oldstatefull;
  bdd     newstatefull;
  bdd     envactions;   
  bdd     sysactions;
};


//*************  domdef     *************************

//  total bdd variable layout description 
struct domdef {
  vector<agtdef>     agt;   
  int                envnum; 
  int                actnum;
  vector<vardef>     var;
  map<string,vardef> varmap;
  vector<string>     varnames;
  int                bddvarnum;
  varrels           *rels;      // both current and next state variable quantifications                       
  domdef(vector<agtdef> a, int en, int acnum, vector<vardef> v,map<string,vardef> vm,
	 vector<string> varn, int bvc)
    { agt = a;  envnum=en; actnum = acnum; var = v; varmap = vm;
    varnames = varn; bddvarnum = bvc;}
  void print();
};



/****************************************************
 *          structure  for representing             *
 *            binary number expresions              *
 ****************************************************/

struct binary {
  int  digitnum;
  bdd *digit;
  binary(int dn, bdd *d)
    { digitnum = dn; digit = d; }
};





/****************************************************
 *          structure  for representing             *
 *       a partitioned transition relation          *
 ****************************************************/


struct relpartition {
  bdd exp;
  bdd varset; // next state vars in support of exp.
  void print();
};

// partitioned transition relation
struct trel {
  int partnum;
  relpartition *p;
  void print();
  trel(int pn,relpartition *pp) {partnum = pn; p = pp;}
};



// *********** function prototypes ********************
domdef *mkdomdef(problem *prob);
void mkbddblocks(domdef *def);
varrels *mkvarrels(domdef *def);
void adjust2order(int *order, domdef *def);


bdd formula2bdd(formula *f, domdef *def) ;
bdd atom2bdd(atom *a, domdef *def) ;

bdd numprop2bdd(numberprop *prop,domdef *def);
binary *numexp2binary(numberexp *num,domdef *def);

bdd int2bdd(int *var,int length,int n)  ;

bdd I(int i, int j, domdef *def) ;
bdd F(string v, int i, int j, domdef *def) ;
bdd ADEF(int agt,domdef *def);
int IND(int i1, int j1, int i2, int j2, domdef *def);
bdd AC(domdef *def);
bdd P(int i, int j, domdef *def) ;
bdd E(int i, int j, domdef *def) ;
bdd IV(set<string> vars,domdef *def) ;
bdd A(int i,int j,domdef *def) ;

trel *mkpartitions(domdef *def, bool adversrial);
int partcmp(relpartition *p1, relpartition *p2);
void sorttrel(trel *T);
void ranjanoptimize(trel *T,int threshold);	      						     






#endif  


