/*========================================================================
                              
                                ** UMOP **
		** Universal Multi-Agent OBDD-based Planner **

            Copyright (C) 1999 by Rune M. Jensen and Manuela M. Veloso
            Computer Science Department, Carnegie Mellon University.
            
    Permission is hereby granted to distribute the UMOP software for 
    research purposes, provided this notice is included with any such 
    distribution. 

      *** WE ARE NOT RESPONSIBLE FOR ANY KIND OF DAMAGE TO YOUR  ***
      *** FILES, DATA, HARDWARE, LOSS OF MONEY, SYSTEM CRASHES   *** 
      *** OR ANY OTHER THING YOU MIGHT COME UP WITH.             ***
      *** - USE THIS PROGRAM OF YOUR OWN FREE WILL !!!           ***

========================================================================*/
/****************************************************** 
 * File  : dissets.hpp  
 * Desc. : Header for Disjoint set ADT
 * Author: Rune M. Jensen
 * Date  : 10/25/00
 ******************************************************/

#ifndef DISSETSHPP
#define DISSETSHPP

#include <vector>
#include <string>
#include <set>
#include <map>
#include "domain.hpp"


// atomic node in the dijoint set datastructure
struct disnode {
  disnode *parent; // pointer to the set parent: the root node is an identifier for the set 
  int size;               // the number of elements in the set (used in unioun)
  int setnum;             // Only assigned for root node, when the final disjoint set is found, 
  disnode() {size = 0;}   // setnum is an identifier value for the set 
};                       



// main datastructure
struct disset {
  vector< set<string> >   disjointset; // Vector of disjoint sets. Assigned after tree computations
  map<string,disnode*> idmap;          // an aux. datastructure mapping each variable to its corresponding disnode 
  int subsetnum;
  disset() {}
  void makeset(string id);
  disnode *find(string id);
  disnode *setunion(disnode *v, disnode *w);
  void mkdisjointsets();               // should only be executed once, when the final disjoint set has been generated
};

#endif  
