/*========================================================================
                              
                                ** UMOP **
		** Universal Multi-Agent OBDD-based Planner **

            Copyright (C) 1999 by Rune M. Jensen and Manuela M. Veloso
            Computer Science Department, Carnegie Mellon University.
            
    Permission is hereby granted to distribute the UMOP software for 
    research purposes, provided this notice is included with any such 
    distribution. 

      *** WE ARE NOT RESPONSIBLE FOR ANY KIND OF DAMAGE TO YOUR  ***
      *** FILES, DATA, HARDWARE, LOSS OF MONEY, SYSTEM CRASHES   *** 
      *** OR ANY OTHER THING YOU MIGHT COME UP WITH.             ***
      *** - USE THIS PROGRAM OF YOUR OWN FREE WILL !!!           ***

========================================================================*/
/****************************************************** 
 * File  : doamin.hpp   
 * Desc. : Domain theory representation for NADL-domains,
 * Author: Rune M. Jensen
 * Date  : 3/3/99
 ******************************************************/

#ifndef DOMAINHPP
#define DOMAINHPP

#include "common.hpp"
#include <set>
#include <list>
#include <string>

/****************************************************
 *                Number expressions                *
 ****************************************************/

enum relop {ro_lt,ro_gt,ro_eq,ro_ne};
enum exptype {nt_number, nt_plus, nt_minus, nt_var};
 

struct numberexp {
  exptype type;
  int number;
  numberexp  *left;
  numberexp  *right;
  string var;
  numberexp(exptype t,int n,numberexp  *l,numberexp  *r,string v)
    {type = t; number = n; left = l; right = r; var = v;}
  numberexp() {}
  void minus2plus();
  void print();
};

struct numberprop {
  relop op;
  numberexp *left;
  numberexp *right;
  numberprop(relop o,numberexp *l,numberexp *r) 
    {op = o; left = l; right = r;}
  numberprop() {}
  void minus2plus();
  void print();
};



/****************************************************
 *                       atoms                      *
 ****************************************************/

enum atomtype {at_numprop, at_boolvar};

struct atom {
  atomtype type;
  numberprop *prop;
  string var;
  atom(atomtype t,numberprop *p,string v)
    {type = t; prop = p; var = v;}
  atom() {};
  void print();
};


/****************************************************
 *               Fomula representation              *
 ****************************************************/


enum formulatype {ft_atom,ft_neg,ft_and,ft_or,ft_impl,
              ft_biimpl,ft_ite, ft_true, ft_false, ft_paren};

struct formula {
  formulatype type;
  atom *atomic;
  formula  *f1;
  formula  *f2;
  formula  *f3;
  formula(formulatype t, atom *a, formula  *f1c, formula  *f2c, formula  *f3c)
    {type = t; atomic = a; f1 = f1c; f2 = f2c; f3 = f3c;}
  formula() {}
  void print();
};


/****************************************************
 *               Action representation              *
 ****************************************************/

struct action {
  string       name;
  set<string>  con;
  formula     *pre;
  formula     *eff;
  action(string n, set<string> *c, formula *p, formula *e)
    {name = n; con = *c; pre = p; eff = e;}
  action() {}
  void print();
}; 


/****************************************************
 *            agent  representation                 *
 ****************************************************/

struct agent {
  string           name;
  list<action>     actions;
  agent(string n, list<action> *a)
    {name = n; actions = *a;}
  agent() {}
  void print();
};


/****************************************************
 *       varlst, vartype representation             
 ****************************************************/

enum vtype {vt_bool, vt_scalar};

struct vartype {
  vtype type;
  int   range;
  vartype(vtype t, int r)
    {type = t; range = r;}
  vartype() {}
  void print();
};



struct vardecl {
  vartype *type;
  set<string> ids;
  vardecl(vartype *t, set<string> *i)
    {type = t; ids = *i;}
  vardecl() {}
  void print();
};


/****************************************************
 *               Problem representation             *
 ****************************************************/

struct problem {
  list<vardecl> vars;
  list<agent>   sys;
  list<agent>   env;
  formula        *init;
  formula        *goal;
  problem(list<vardecl> *v, list<agent> *s, list<agent> *e, formula *i, formula *g)
    {vars = *v; sys = *s; env = *e; init = i; goal = g;}
  problem() {}
  void print();
}; 


// *********** function prototypes ********************
//idlst *currentvars(action *act,idlst *curvarnames) ;
//idlst *formula2curvars(formula *f, idlst *curvarnames) ;
//idlst *atom2curvars(atom *a, idlst *curvarnames) ;
//idlst *numprop2curvars(numberprop *prop, idlst *curvarnames);
//idlst *numexp2curvars(numberexp *num, idlst *curvarnames);

int yyparse();


#endif
