Number                  [0-9]+
Id                      [a-zA-Z_][0-9A-Za-z_]*'?

%{
/****************************************************** 
 * File  : nadl.l
 * Desc. : lex file for NADL Language 
 * Author: Rune M. Jensen 
 * Date  : 2/21/99, CS,CMU
 ******************************************************/

#include "y.tab.h"

int lineno = 1;

%}

%%
"%".*                   /* ignore comments, (rest of line from %) */ ;
[\t ]+                  /* ignore whitespace */ ;
\n                      { lineno++; }
"system"                { return(SYSTEM); }
"environment"           { return(ENVIRONMENT); }
"variables"             { return(VARIABLES); }
"initially"             { return(INITIALLY); }
"goal"                  { return(GOAL); }
"bool"                  { return(BOOL); }
"nat"                   { return(SCALAR); }
"agt:"                  { return(AGT); }
"pre:"                  { return(PRE); }
"eff:"                  { return(EFF); }
"con:"                  { return(VAR); }
"true"                  { return(TRUE); }
"false"                 { return(FALSE); }
"SYSTEM"                { return(SYSTEM); }
"ENVIRONMENT"           { return(ENVIRONMENT); }
"VARIABLES"             { return(VARIABLES); }
"INITIALLY"             { return(INITIALLY); }
"GOAL"                  { return(GOAL); }
"BOOL"                  { return(BOOL); }
"NAT"                   { return(SCALAR); }
"AGT:"                  { return(AGT); }
"PRE:"                  { return(PRE); }
"EFF:"                  { return(EFF); }
"CON:"                  { return(VAR); }
"TRUE"                  { return(TRUE); }
"FALSE"                 { return(FALSE); }
"<=>"                   { return(BIIMPL); }
"=>"                    { return(IMPL); }
"->"                    { return(ARROW); }
"/\\"                   { return(AND); }
"\\/"                   { return(OR); }
"<>"                    { return(NE); }
"~"                     { return('~'); }
"="                     { return('='); }
">"                     { return('>'); }
"<"                     { return('<'); }
"+"                     { return('+'); }
"-"                     { return('-'); }
","                     { return(','); }
"("                     { return('('); }
")"                     { return(')'); }
{Id}                    { return(ID); }
{Number}                { return(NUMBER); }
%%



